[llvm-commits] [llvm] r57687 - in /llvm/trunk: include/llvm/InlineAsm.h include/llvm/Target/TargetLowering.h lib/CodeGen/SelectionDAG/SelectionDAGBuild.cpp lib/CodeGen/SelectionDAG/TargetLowering.cpp lib/VMCore/InlineAsm.cpp

Chris Lattner sabre at nondot.org
Fri Oct 17 09:47:48 PDT 2008


Author: lattner
Date: Fri Oct 17 11:47:46 2008
New Revision: 57687

URL: http://llvm.org/viewvc/llvm-project?rev=57687&view=rev
Log:
Keep track of *which* input constraint matches an output
constraint.  Reject asms where an output has multiple
input constraints tied to it.

Modified:
    llvm/trunk/include/llvm/InlineAsm.h
    llvm/trunk/include/llvm/Target/TargetLowering.h
    llvm/trunk/lib/CodeGen/SelectionDAG/SelectionDAGBuild.cpp
    llvm/trunk/lib/CodeGen/SelectionDAG/TargetLowering.cpp
    llvm/trunk/lib/VMCore/InlineAsm.cpp

Modified: llvm/trunk/include/llvm/InlineAsm.h
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/include/llvm/InlineAsm.h?rev=57687&r1=57686&r2=57687&view=diff

==============================================================================
--- llvm/trunk/lib/CodeGen/SelectionDAG/SelectionDAGBuild.cpp (original)
+++ llvm/trunk/lib/CodeGen/SelectionDAG/SelectionDAGBuild.cpp Fri Oct 17 11:47:46 2008
@@ -4491,7 +4491,7 @@
     
     // If there is an input constraint that matches this, we need to reserve 
     // the input register so no other inputs allocate to it.
-    isInReg = OpInfo.hasMatchingInput;
+    isInReg = OpInfo.hasMatchingInput();
     break;
   case InlineAsm::isInput:
     isInReg = true;
@@ -4562,7 +4562,7 @@
     // the constraint, so we have to pick a register to pin the input/output to.
     // If it isn't a matched constraint, go ahead and create vreg and let the
     // regalloc do its thing.
-    if (!OpInfo.hasMatchingInput) {
+    if (!OpInfo.hasMatchingInput()) {
       RegVT = *PhysReg.second->vt_begin();
       if (OpInfo.ConstraintVT == MVT::Other)
         ValueVT = RegVT;
@@ -4863,7 +4863,7 @@
     case InlineAsm::isInput: {
       SDValue InOperandVal = OpInfo.CallOperand;
       
-      if (OpInfo.isMatchingConstraint()) {   // Matching constraint?
+      if (OpInfo.isMatchingInputConstraint()) {   // Matching constraint?
         // If this is required to match an output register we have already set,
         // just use its register.
         unsigned OperandNo = OpInfo.getMatchedOperand();

Modified: llvm/trunk/lib/CodeGen/SelectionDAG/TargetLowering.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/CodeGen/SelectionDAG/TargetLowering.cpp?rev=57687&r1=57686&r2=57687&view=diff

==============================================================================
--- llvm/trunk/lib/VMCore/InlineAsm.cpp (original)
+++ llvm/trunk/lib/VMCore/InlineAsm.cpp Fri Oct 17 11:47:46 2008
@@ -57,7 +57,7 @@
   // Initialize
   Type = isInput;
   isEarlyClobber = false;
-  hasMatchingInput = false;
+  MatchingInput = -1;
   isCommutative = false;
   isIndirect = false;
   
@@ -127,8 +127,13 @@
           Type != isInput)
         return true;  // Invalid constraint number.
       
+      // If Operand N already has a matching input, reject this.  An output
+      // can't be constrained to the same value as multiple inputs.
+      if (ConstraintsSoFar[N].hasMatchingInput())
+        return true;
+      
       // Note that operand #n has a matching input.
-      ConstraintsSoFar[N].hasMatchingInput = true;
+      ConstraintsSoFar[N].MatchingInput = ConstraintsSoFar.size();
     } else {
       // Single letter constraint.
       Codes.push_back(std::string(I, I+1));





More information about the llvm-commits mailing list
--- llvm/trunk/include/llvm/InlineAsm.h (original)
+++ llvm/trunk/include/llvm/InlineAsm.h Fri Oct 17 11:47:46 2008
@@ -79,9 +79,15 @@
     /// read.  This is only ever set for an output operand.
     bool isEarlyClobber; 
     
-    /// hasMatchingInput - This is set to true for an output constraint iff
-    /// there is an input constraint that is required to match it (e.g. "0").
-    bool hasMatchingInput;
+    /// MatchingInput - If this is not -1, this is an output constraint where an
+    /// input constraint is required to match it (e.g. "0").  The value is the
+    /// constraint number that matches this one (for example, if this is
+    /// constraint #0 and constraint #4 has the value "0", this will be 4).
+    signed char MatchingInput;
+    
+    /// hasMatchingInput - Return true if this is an output constraint that has
+    /// a matching input constraint.
+    bool hasMatchingInput() const { return MatchingInput != -1; }
     
     /// isCommutative - This is set to true for a constraint that is commutative
     /// with the next operand.

Modified: llvm/trunk/include/llvm/Target/TargetLowering.h
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/include/llvm/Target/TargetLowering.h?rev=57687&r1=57686&r2=57687&view=diff

==============================================================================
--- llvm/trunk/include/llvm/Target/TargetLowering.h (original)
+++ llvm/trunk/include/llvm/Target/TargetLowering.h Fri Oct 17 11:47:46 2008
@@ -1197,9 +1197,9 @@
     /// ConstraintVT - The ValueType for the operand value.
     MVT ConstraintVT;
     
-    /// isMatchingConstraint - Return true of this is an input operand that is a
-    /// matching constraint like "4".
-    bool isMatchingConstraint() const;
+    /// isMatchingInputConstraint - Return true of this is an input operand that
+    /// is a matching constraint like "4".
+    bool isMatchingInputConstraint() const;
     
     /// getMatchedOperand - If this is an input matching constraint, this method
     /// returns the output operand it matches.

Modified: llvm/trunk/lib/CodeGen/SelectionDAG/SelectionDAGBuild.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/CodeGen/SelectionDAG/SelectionDAGBuild.cpp?rev=57687&r1=57686&r2=57687&view=diff

==============================================================================
--- llvm/trunk/lib/CodeGen/SelectionDAG/TargetLowering.cpp (original)
+++ llvm/trunk/lib/CodeGen/SelectionDAG/TargetLowering.cpp Fri Oct 17 11:47:46 2008
@@ -1962,9 +1962,9 @@
 //===----------------------------------------------------------------------===//
 // Constraint Selection.
 
-/// isMatchingConstraint - Return true of this is an input operand that is a
-/// matching constraint like "4".
-bool TargetLowering::AsmOperandInfo::isMatchingConstraint() const {
+/// isMatchingInputConstraint - Return true of this is an input operand that is
+/// a matching constraint like "4".
+bool TargetLowering::AsmOperandInfo::isMatchingInputConstraint() const {
   assert(!ConstraintCode.empty() && "No known constraint!");
   return isdigit(ConstraintCode[0]);
 }

Modified: llvm/trunk/lib/VMCore/InlineAsm.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/VMCore/InlineAsm.cpp?rev=57687&r1=57686&r2=57687&view=diff

==============================================================================